@@ -50,45 +50,8 @@
 #include <X11/extensions/Xfixes.h>
 #include <X11/extensions/Xrender.h>
 
-#if COMPOSITE_MAJOR > 0 || COMPOSITE_MINOR >= 2
-#define HAVE_NAME_WINDOW_PIXMAP 1
-#endif
-
-#if COMPOSITE_MAJOR > 0 || COMPOSITE_MINOR >= 3
-#define HAVE_COW 1
-#else
-/* Don't have a cow man...HAAHAAHAA */
-#endif
-
 #define USE_IDLE_REPAINT 1
 
-#ifdef __GNUC__
-#define UNUSED_VARIABLE __attribute__ ((unused))
-#else
-#define UNUSED_VARIABLE
-#endif
-
-#ifdef HAVE_COMPOSITE_EXTENSIONS
-static inline gboolean
-composite_at_least_version (MetaDisplay *display,
-                            int maj, int min)
-{
-  static int major = -1;
-  static int minor = -1;
-
-  if (major == -1)
-    meta_display_get_compositor_version (display, &major, &minor);
-
-  return (major > maj || (major == maj && minor >= min));
-}
-
-#define have_name_window_pixmap(display) \
-  composite_at_least_version (display, 0, 2)
-#define have_cow(display) \
-  composite_at_least_version (display, 0, 3)
-
-#endif
-
 typedef enum _MetaCompWindowType
 {
   META_COMP_WINDOW_NORMAL,
@@ -186,7 +149,6 @@ typedef struct _MetaCompWindow
   Window id;
   XWindowAttributes attrs;
 
-#ifdef HAVE_NAME_WINDOW_PIXMAP
   Pixmap back_pixmap;
 
   /* When the window is shaded back_pixmap will be replaced with the pixmap
@@ -194,7 +156,6 @@ typedef struct _MetaCompWindow
      so that we can still see what the window looked like when it is needed
      for the _get_window_pixmap function */
   Pixmap shaded_back_pixmap;
-#endif
 
   int mode;
 
@@ -905,6 +866,13 @@ window_has_shadow (MetaCompWindow *cw)
      as the frame might be the reason the window is shaped */
   if (cw->window)
     {
+      /* Do not add shadows for maximized windows */
+      if (meta_window_is_maximized (cw->window))
+        {
+          meta_verbose ("Window has no shadow because it is maximized\n");
+          return FALSE;
+        }
+
       if (meta_window_get_frame (cw->window)) {
         meta_verbose ("Window has shadow because it has a frame\n");
         return TRUE;
@@ -1061,21 +1029,21 @@ get_window_picture (MetaCompWindow *cw)
   XRenderPictureAttributes pa;
   XRenderPictFormat *format;
   Drawable draw;
+  int error_code;
 
   draw = cw->id;
 
   meta_error_trap_push (display);
 
-#ifdef HAVE_NAME_WINDOW_PIXMAP
-  if (have_name_window_pixmap (display))
-    {
-      if (cw->back_pixmap == None)
-        cw->back_pixmap = XCompositeNameWindowPixmap (xdisplay, cw->id);
+  if (cw->back_pixmap == None)
+    cw->back_pixmap = XCompositeNameWindowPixmap (xdisplay, cw->id);
 
-      if (cw->back_pixmap != None)
-        draw = cw->back_pixmap;
-    }
-#endif
+  error_code = meta_error_trap_pop_with_return (display, FALSE);
+  if (error_code != 0)
+    cw->back_pixmap = None;
+
+  if (cw->back_pixmap != None)
+    draw = cw->back_pixmap;
 
   format = get_window_format (cw);
   if (format)
@@ -1084,13 +1052,13 @@ get_window_picture (MetaCompWindow *cw)
 
       pa.subwindow_mode = IncludeInferiors;
 
+      meta_error_trap_push (display);
       pict = XRenderCreatePicture (xdisplay, draw, format, CPSubwindowMode, &pa);
       meta_error_trap_pop (display, FALSE);
 
       return pict;
     }
 
-  meta_error_trap_pop (display, FALSE);
   return None;
 }
 
@@ -1143,8 +1111,7 @@ paint_windows (MetaScreen   *screen,
   MetaCompScreen *info = meta_screen_get_compositor_data (screen);
   Display *xdisplay = meta_display_get_xdisplay (display);
   GList *index, *last;
-  int screen_width, screen_height, UNUSED_VARIABLE screen_number;
-  Window UNUSED_VARIABLE xroot;
+  int screen_width, screen_height;
   MetaCompWindow *cw;
   XserverRegion paint_region, desktop_region;
 
@@ -1154,8 +1121,6 @@ paint_windows (MetaScreen   *screen,
     }
 
   meta_screen_get_size (screen, &screen_width, &screen_height);
-  screen_number = meta_screen_get_screen_number (screen);
-  xroot = meta_screen_get_xroot (screen);
 
   if (region == None)
     {
@@ -1233,22 +1198,10 @@ paint_windows (MetaScreen   *screen,
         {
           int x, y, wid, hei;
 
-#ifdef HAVE_NAME_WINDOW_PIXMAP
-          if (have_name_window_pixmap (display))
-            {
-              x = cw->attrs.x;
-              y = cw->attrs.y;
-              wid = cw->attrs.width + cw->attrs.border_width * 2;
-              hei = cw->attrs.height + cw->attrs.border_width * 2;
-            }
-          else
-#endif
-            {
-              x = cw->attrs.x + cw->attrs.border_width;
-              y = cw->attrs.y + cw->attrs.border_width;
-              wid = cw->attrs.width;
-              hei = cw->attrs.height;
-            }
+          x = cw->attrs.x;
+          y = cw->attrs.y;
+          wid = cw->attrs.width + cw->attrs.border_width * 2;
+          hei = cw->attrs.height + cw->attrs.border_width * 2;
 
           XFixesSetPictureClipRegion (xdisplay, root_buffer,
                                       0, 0, paint_region);
@@ -1324,22 +1277,11 @@ paint_windows (MetaScreen   *screen,
           if (cw->mode == WINDOW_ARGB)
             {
               int x, y, wid, hei;
-#ifdef HAVE_NAME_WINDOW_PIXMAP
-              if (have_name_window_pixmap (display))
-                {
-                  x = cw->attrs.x;
-                  y = cw->attrs.y;
-                  wid = cw->attrs.width + cw->attrs.border_width * 2;
-                  hei = cw->attrs.height + cw->attrs.border_width * 2;
-                }
-              else
-#endif
-                {
-                  x = cw->attrs.x + cw->attrs.border_width;
-                  y = cw->attrs.y + cw->attrs.border_width;
-                  wid = cw->attrs.width;
-                  hei = cw->attrs.height;
-                }
+
+              x = cw->attrs.x;
+              y = cw->attrs.y;
+              wid = cw->attrs.width + cw->attrs.border_width * 2;
+              hei = cw->attrs.height + cw->attrs.border_width * 2;
 
               XRenderComposite (xdisplay, PictOpOver, cw->picture,
                                 cw->alpha_pict, root_buffer, 0, 0, 0, 0,
@@ -1480,13 +1422,18 @@ add_damage (MetaScreen     *screen,
 
   /*  dump_xserver_region ("add_damage", display, damage); */
 
-  if (info != NULL && info->all_damage)
+  if (info != NULL)
     {
-      XFixesUnionRegion (xdisplay, info->all_damage, info->all_damage, damage);
-      XFixesDestroyRegion (xdisplay, damage);
+      if (info->all_damage)
+        {
+          XFixesUnionRegion (xdisplay, info->all_damage, info->all_damage, damage);
+          XFixesDestroyRegion (xdisplay, damage);
+        }
+      else
+        {
+          info->all_damage = damage;
+        }
     }
-  else
-    info->all_damage = damage;
 
 #ifdef USE_IDLE_REPAINT
   add_repair (display);
@@ -1551,23 +1498,18 @@ free_win (MetaCompWindow *cw,
   Display *xdisplay = meta_display_get_xdisplay (display);
   MetaCompScreen *info = meta_screen_get_compositor_data (cw->screen);
 
-#ifdef HAVE_NAME_WINDOW_PIXMAP
-  if (have_name_window_pixmap (display))
+  /* See comment in map_win */
+  if (cw->back_pixmap && destroy)
     {
-      /* See comment in map_win */
-      if (cw->back_pixmap && destroy)
-        {
-          XFreePixmap (xdisplay, cw->back_pixmap);
-          cw->back_pixmap = None;
-        }
+      XFreePixmap (xdisplay, cw->back_pixmap);
+      cw->back_pixmap = None;
+    }
 
-      if (cw->shaded_back_pixmap && destroy)
-        {
-          XFreePixmap (xdisplay, cw->shaded_back_pixmap);
-          cw->shaded_back_pixmap = None;
-        }
+  if (cw->shaded_back_pixmap && destroy)
+    {
+      XFreePixmap (xdisplay, cw->shaded_back_pixmap);
+      cw->shaded_back_pixmap = None;
     }
-#endif
 
   if (cw->picture)
     {
@@ -1641,7 +1583,6 @@ map_win (MetaDisplay *display,
   if (cw == NULL)
     return;
 
-#ifdef HAVE_NAME_WINDOW_PIXMAP
   /* The reason we deallocate this here and not in unmap
      is so that we will still have a valid pixmap for
      whenever the window is unmapped */
@@ -1656,7 +1597,6 @@ map_win (MetaDisplay *display,
       XFreePixmap (xdisplay, cw->shaded_back_pixmap);
       cw->shaded_back_pixmap = None;
     }
-#endif
 
   cw->attrs.map_state = IsViewable;
   cw->damaged = FALSE;
@@ -1827,7 +1767,7 @@ add_win (MetaScreen *screen,
 
   if (xwindow == info->output)
     return;
-    
+
   /* If already added, ignore */
   if (find_window_for_screen (screen, xwindow) != NULL)
     return;
@@ -1850,11 +1790,8 @@ add_win (MetaScreen *screen,
 
   XSelectInput (xdisplay, xwindow, event_mask);
 
-
-#ifdef HAVE_NAME_WINDOW_PIXMAP
   cw->back_pixmap = None;
   cw->shaded_back_pixmap = None;
-#endif
 
   cw->damaged = FALSE;
   cw->shaped = is_shaped (display, xwindow);
@@ -2036,32 +1973,28 @@ resize_win (MetaCompWindow *cw,
 
   if (cw->attrs.width != width || cw->attrs.height != height)
     {
-#ifdef HAVE_NAME_WINDOW_PIXMAP
-      if (have_name_window_pixmap (display))
+      if (cw->shaded_back_pixmap)
         {
-          if (cw->shaded_back_pixmap)
-            {
-              XFreePixmap (xdisplay, cw->shaded_back_pixmap);
-              cw->shaded_back_pixmap = None;
-            }
+          XFreePixmap (xdisplay, cw->shaded_back_pixmap);
+          cw->shaded_back_pixmap = None;
+        }
 
-          if (cw->back_pixmap)
+      if (cw->back_pixmap)
+        {
+          /* If the window is shaded, we store the old backing pixmap
+             so we can return a proper image of the window */
+          if (cw->window && meta_window_is_shaded (cw->window))
             {
-              /* If the window is shaded, we store the old backing pixmap
-                 so we can return a proper image of the window */
-              if (cw->window && meta_window_is_shaded (cw->window))
-                {
-                  cw->shaded_back_pixmap = cw->back_pixmap;
-                  cw->back_pixmap = None;
-                }
-              else
-                {
-                  XFreePixmap (xdisplay, cw->back_pixmap);
-                  cw->back_pixmap = None;
-                }
+              cw->shaded_back_pixmap = cw->back_pixmap;
+              cw->back_pixmap = None;
+            }
+          else
+            {
+              XFreePixmap (xdisplay, cw->back_pixmap);
+              cw->back_pixmap = None;
             }
         }
-#endif
+
       if (cw->picture)
         {
           XRenderFreePicture (xdisplay, cw->picture);
@@ -2498,29 +2431,22 @@ show_overlay_window (MetaScreen *screen,
 {
   MetaDisplay *display = meta_screen_get_display (screen);
   Display *xdisplay = meta_display_get_xdisplay (display);
+  XserverRegion region;
 
-#ifdef HAVE_COW
-  if (have_cow (display))
-    {
-      XserverRegion region;
+  region = XFixesCreateRegion (xdisplay, NULL, 0);
 
-      region = XFixesCreateRegion (xdisplay, NULL, 0);
+  XFixesSetWindowShapeRegion (xdisplay, cow, ShapeBounding, 0, 0, 0);
+  XFixesSetWindowShapeRegion (xdisplay, cow, ShapeInput, 0, 0, region);
 
-      XFixesSetWindowShapeRegion (xdisplay, cow, ShapeBounding, 0, 0, 0);
-      XFixesSetWindowShapeRegion (xdisplay, cow, ShapeInput, 0, 0, region);
+  XFixesDestroyRegion (xdisplay, region);
 
-      XFixesDestroyRegion (xdisplay, region);
-
-      damage_screen (screen);
-    }
-#endif
+  damage_screen (screen);
 }
 
 static void
 hide_overlay_window (MetaScreen *screen,
                      Window      cow)
 {
-#ifdef HAVE_COW
   MetaDisplay *display = meta_screen_get_display (screen);
   Display *xdisplay = meta_display_get_xdisplay (display);
   XserverRegion region;
@@ -2528,7 +2454,6 @@ hide_overlay_window (MetaScreen *screen,
   region = XFixesCreateRegion (xdisplay, NULL, 0);
   XFixesSetWindowShapeRegion (xdisplay, cow, ShapeBounding, 0, 0, region);
   XFixesDestroyRegion (xdisplay, region);
-#endif
 }
 
 static Window
@@ -2540,17 +2465,8 @@ get_output_window (MetaScreen *screen)
 
   xroot = meta_screen_get_xroot (screen);
 
-#ifdef HAVE_COW
-  if (have_cow (display))
-    {
-      output = XCompositeGetOverlayWindow (xdisplay, xroot);
-      XSelectInput (xdisplay, output, ExposureMask);
-    }
-  else
-#endif
-    {
-      output = xroot;
-    }
+  output = XCompositeGetOverlayWindow (xdisplay, xroot);
+  XSelectInput (xdisplay, output, ExposureMask);
 
   return output;
 }
@@ -2687,9 +2603,7 @@ xrender_unmanage_screen (MetaCompositor *compositor,
                                   CompositeRedirectManual);
   meta_screen_unset_cm_selection (screen);
 
-#ifdef HAVE_COW
   XCompositeReleaseOverlayWindow (xdisplay, info->output);
-#endif
 
   g_free (info);
 
@@ -2748,23 +2662,38 @@ xrender_end_move (MetaCompositor *compositor,
 #ifdef HAVE_COMPOSITE_EXTENSIONS
 #endif
 }
+#endif /* 0 */
 
 static void
 xrender_free_window (MetaCompositor *compositor,
                      MetaWindow     *window)
 {
 #ifdef HAVE_COMPOSITE_EXTENSIONS
-  /* FIXME: When an undecorated window is hidden this is called,
-     but the window does not get readded if it is subsequentally shown again
-     See http://bugzilla.gnome.org/show_bug.cgi?id=504876
+  MetaCompositorXRender *xrc;
+  MetaFrame *frame;
+  Window xwindow;
 
-     I don't *think* theres any need for this call anyway, leaving it out
-     does not seem to cause any side effects so far, but I should check with
-     someone who understands more. */
-  /* destroy_win (compositor->display, window->xwindow, FALSE); */
+  xrc = (MetaCompositorXRender *) compositor;
+  frame = meta_window_get_frame (window);
+  xwindow = None;
+
+  if (frame)
+    {
+      xwindow = meta_frame_get_xwindow (frame);
+    }
+  else
+    {
+      /* FIXME: When an undecorated window is hidden this is called, but the
+       * window does not get readded if it is subsequentally shown again. See:
+       * http://bugzilla.gnome.org/show_bug.cgi?id=504876
+       */
+      /* xwindow = meta_window_get_xwindow (window); */
+    }
+
+  if (xwindow != None)
+    destroy_win (xrc->display, xwindow, FALSE);
 #endif
 }
-#endif /* 0 */
 
 static void
 xrender_process_event (MetaCompositor *compositor,
@@ -2855,17 +2784,10 @@ xrender_get_window_pixmap (MetaCompositor *compositor,
   if (cw == NULL)
     return None;
 
-#ifdef HAVE_NAME_WINDOW_PIXMAP
-  if (have_name_window_pixmap (meta_window_get_display (window)))
-    {
-      if (meta_window_is_shaded (window))
-        return cw->shaded_back_pixmap;
-      else
-        return cw->back_pixmap;
-    }
+  if (meta_window_is_shaded (window))
+    return cw->shaded_back_pixmap;
   else
-#endif
-    return None;
+    return cw->back_pixmap;
 #endif
 }
 
@@ -3011,6 +2933,38 @@ xrender_set_active_window (MetaCompositor *compositor,
 #endif
 }
 
+static void
+xrender_maximize_window (MetaCompositor *compositor,
+                         MetaWindow     *window)
+{
+#ifdef HAVE_COMPOSITE_EXTENSIONS
+  MetaFrame *frame = meta_window_get_frame (window);
+  Window xid = frame ? meta_frame_get_xwindow (frame) : meta_window_get_xwindow (window);
+  MetaCompWindow *cw = find_window_in_display (meta_window_get_display (window), xid);
+
+  if (!cw)
+    return;
+
+  cw->needs_shadow = window_has_shadow (cw);
+#endif
+}
+
+static void
+xrender_unmaximize_window (MetaCompositor *compositor,
+                           MetaWindow     *window)
+{
+#ifdef HAVE_COMPOSITE_EXTENSIONS
+  MetaFrame *frame = meta_window_get_frame (window);
+  Window xid = frame ? meta_frame_get_xwindow (frame) : meta_window_get_xwindow (window);
+  MetaCompWindow *cw = find_window_in_display (meta_window_get_display (window), xid);
+
+  if (!cw)
+    return;
+
+  cw->needs_shadow = window_has_shadow (cw);
+#endif
+}
+
 static MetaCompositor comp_info = {
   xrender_destroy,
   xrender_manage_screen,
@@ -3020,7 +2974,10 @@ static MetaCompositor comp_info = {
   xrender_set_updates,
   xrender_process_event,
   xrender_get_window_pixmap,
-  xrender_set_active_window
+  xrender_set_active_window,
+  xrender_free_window,
+  xrender_maximize_window,
+  xrender_unmaximize_window,
 };
 
 MetaCompositor *
