@@ -67,7 +67,6 @@
 #endif
 
 #define META_MINIMIZE_ANIMATION_LENGTH 0.25
-#define META_UNMINIMIZE_ANIMATION_LENGTH 0.25
 #define META_SHADE_ANIMATION_LENGTH 0.2
 
 #include <string.h>
@@ -99,9 +98,6 @@ typedef struct
   MetaRectangle start_rect;
   MetaRectangle end_rect;
 
-  MetaEffectFinished finished;
-  gpointer finished_data;
-
 } BoxAnimationContext;
 
 /**
@@ -153,9 +149,7 @@ static void
 draw_box_animation (MetaScreen     *screen,
                     MetaRectangle  *initial_rect,
                     MetaRectangle  *destination_rect,
-                    double          seconds_duration,
-                    MetaEffectFinished finished,
-                    gpointer        finished_data);
+                    double          seconds_duration);
 
 /**
  * Creates an effect.
@@ -179,13 +173,17 @@ create_effect (MetaEffectType      type,
 }
 
 /**
- * Destroys an effect.
+ * Destroys an effect.  If the effect has a "finished" hook, it will be
+ * called before cleanup.
  *
  * \param effect  The effect.
  */
 static void
 effect_free (MetaEffect *effect)
 {
+  if (effect->priv->finished)
+    effect->priv->finished (effect->priv->finished_data);
+
   g_free (effect->priv);
   g_free (effect);
 }
@@ -316,23 +314,6 @@ update_wireframe_window (MetaDisplay         *display,
 }
 #endif
 
-/**
- * A hack to force the X server to synchronize with the
- * graphics hardware.
- */
-static void
-graphics_sync (BoxAnimationContext *context)
-{
-  XImage *image;
-
-  image = XGetImage (context->screen->display->xdisplay,
-                     context->screen->xroot,
-                     0, 0, 1, 1,
-                     AllPlanes, ZPixmap);
-
-  XDestroyImage (image);
-}
-
 static gboolean
 effects_draw_box_animation_timeout (BoxAnimationContext *context)
 {
@@ -383,10 +364,6 @@ effects_draw_box_animation_timeout (BoxAnimationContext *context)
                  context->gc);
 #endif /* !HAVE_SHAPE */
 
-      graphics_sync (context);
-
-      context->finished(context->finished_data);
-
       g_free (context);
       return FALSE;
     }
@@ -425,7 +402,7 @@ effects_draw_box_animation_timeout (BoxAnimationContext *context)
 #endif /* !HAVE_SHAPE */
 
   /* kick changes onto the server */
-  graphics_sync (context);
+  XFlush (context->screen->display->xdisplay);
 
   return TRUE;
 }
@@ -434,9 +411,7 @@ void
 draw_box_animation (MetaScreen     *screen,
                     MetaRectangle  *initial_rect,
                     MetaRectangle  *destination_rect,
-                    double          seconds_duration,
-                    MetaEffectFinished finished,
-                    gpointer        finished_data)
+                    double          seconds_duration)
 {
   BoxAnimationContext *context;
 
@@ -461,9 +436,6 @@ draw_box_animation (MetaScreen     *screen,
   context->start_rect = *initial_rect;
   context->end_rect = *destination_rect;
 
-  context->finished = finished;
-  context->finished_data = finished_data;
-
 #ifdef HAVE_SHAPE
 
   attrs.override_redirect = True;
@@ -726,17 +698,7 @@ run_default_effect_handler (MetaEffect *effect)
        draw_box_animation (effect->window->screen,
                      &(effect->u.minimize.window_rect),
                      &(effect->u.minimize.icon_rect),
-                     META_MINIMIZE_ANIMATION_LENGTH,
-                     effect->priv->finished,
-                     effect->priv->finished_data);
-       break;
-    case META_EFFECT_UNMINIMIZE:
-       draw_box_animation (effect->window->screen,
-                     &(effect->u.minimize.icon_rect),
-                     &(effect->u.minimize.window_rect),
-                     META_UNMINIMIZE_ANIMATION_LENGTH,
-                     effect->priv->finished,
-                     effect->priv->finished_data);
+                     META_MINIMIZE_ANIMATION_LENGTH);
        break;
 
     default:
@@ -747,16 +709,8 @@ run_default_effect_handler (MetaEffect *effect)
 static void
 run_handler (MetaEffect *effect)
 {
-  /* If effects are disabled just run the finished function */
   if (meta_prefs_get_mate_animations ())
-  {
     run_default_effect_handler (effect);
-  }
-  else
-  {
-    if (effect->priv->finished)
-      effect->priv->finished(effect->priv->finished_data);
-  }
 
   effect_free (effect);
 }
