@@ -36,7 +36,6 @@
 
 #include <config.h>
 #include "window.h"
-#include "effects.h"
 #include "screen-private.h"
 #include "util.h"
 #include "stack.h"
@@ -139,6 +138,9 @@ struct _MetaWindow
   guint maximize_vertically_after_placement : 1;
   guint minimize_after_placement : 1;
 
+  /* Whether to move after placement */
+  guint move_after_placement : 1;
+
   /* The current or requested tile mode. If maximized_vertically is true,
    * this is the current mode. If not, it is the mode which will be
    * requested after the window grab is released */
@@ -184,9 +186,6 @@ struct _MetaWindow
   guint was_minimized : 1;
   guint tab_unminimized : 1;
 
-  /* Whether there is a pending effect */
-  MetaEffectType effect_pending;
-
   /* Whether the window is mapped; actual server-side state
    * see also unmaps_pending
    */
