@@ -389,6 +389,9 @@ meta_workspace_activate_with_focus (MetaWorkspace *workspace,
   /* Note that old can be NULL; e.g. when starting up */
   old = workspace->screen->active_workspace;
   
+  /* Save old workspace, to be able to switch back. */
+  workspace->screen->prev_workspace = old;
+
   workspace->screen->active_workspace = workspace;
 
   set_active_space_hint (workspace->screen);
@@ -807,6 +810,8 @@ meta_motion_direction_to_string (MetaMotionDirection direction)
       return "Left";
     case META_MOTION_RIGHT:
       return "Right";
+    case META_MOTION_PREV:
+      return "Previous";
     }
 
   return "Unknown";
@@ -847,6 +852,8 @@ meta_workspace_get_neighbor (MetaWorkspace      *workspace,
     case META_MOTION_DOWN:
       layout.current_row += 1;
       break;
+    case META_MOTION_PREV:
+      break;
     }
 
   /* LEFT */
@@ -923,6 +930,8 @@ meta_workspace_get_neighbor (MetaWorkspace      *workspace,
         if (wrap == META_WRAP_TOROIDAL)
 	  layout.current_col = layout.current_col < layout.cols - 1 ? layout.current_col + 1 : 0;
         break;
+      case META_MOTION_PREV:
+        break;
       }
 
   i = layout.grid[layout.current_row * layout.cols + layout.current_col];
