@@ -52,6 +52,16 @@ struct _MetaUI {
 
 void meta_ui_init(int* argc, char*** argv)
 {
+  /* As of 2.91.7, Gdk uses XI2 by default, which conflicts with the
+   * direct X calls we use - in particular, events caused by calls to
+   * XGrabPointer/XGrabKeyboard are no longer understood by GDK, while
+   * GDK will no longer generate the core XEvents we process.
+   * So at least for now, enforce the previous behavior.
+   */
+#if GTK_CHECK_VERSION(3, 0, 0)
+  gdk_disable_multidevice ();
+#endif
+
 	if (!gtk_init_check (argc, argv))
 	{
 		meta_fatal ("Unable to open X display %s\n", XDisplayName (NULL));
@@ -85,6 +95,7 @@ maybe_redirect_mouse_event (XEvent *xevent)
   MetaUI *ui;
 #if GTK_CHECK_VERSION (3, 0, 0)
   GdkDeviceManager *gmanager;
+  GdkDevice *gdevice;
   GdkEvent *gevent;
 #else
   GdkEvent gevent;
@@ -114,19 +125,24 @@ maybe_redirect_mouse_event (XEvent *xevent)
   if (!ui)
     return FALSE;
 
-#if GTK_CHECK_VERSION (3, 0, 0)
   gdk_window = gdk_x11_window_lookup_for_display (gdisplay, window);
-#else
-  gdk_window = gdk_window_lookup_for_display (gdisplay, window);
-#endif
   if (gdk_window == NULL)
     return FALSE;
 
+#if GTK_CHECK_VERSION (3, 0, 0)
+  gmanager = gdk_display_get_device_manager (gdisplay);
+  gdevice = gdk_device_manager_get_client_pointer (gmanager);
+#endif
+
   /* If GDK already thinks it has a grab, we better let it see events; this
    * is the menu-navigation case and events need to get sent to the appropriate
    * (client-side) subwindow for individual menu items.
    */
+#if GTK_CHECK_VERSION (3, 0, 0)
+  if (gdk_display_device_is_grabbed (gdisplay, gdevice))
+#else
   if (gdk_display_pointer_is_grabbed (gdisplay))
+#endif
     return FALSE;
 
 #if !GTK_CHECK_VERSION (3, 0, 0)
@@ -235,8 +251,8 @@ maybe_redirect_mouse_event (XEvent *xevent)
 
   /* If we've gotten here, we've filled in the gdk_event and should send it on */
 #if GTK_CHECK_VERSION (3, 0, 0)
-  gmanager = gdk_display_get_device_manager (gdisplay);
-  gdk_event_set_device (gevent, gdk_device_manager_get_client_pointer (gmanager));  gtk_main_do_event (gevent);
+  gdk_event_set_device (gevent, gdevice);
+  gtk_main_do_event (gevent);
   gdk_event_free (gevent);
 #else
   gtk_main_do_event (&gevent);
@@ -313,7 +329,14 @@ meta_ui_new (Display *xdisplay,
 
   g_assert (xdisplay == GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()));
   ui->frames = meta_frames_new (XScreenNumberOfScreen (screen));
+#if GTK_CHECK_VERSION (3, 0, 0)
+  /* This does not actually show any widget. MetaFrames has been hacked so
+   * that showing it doesn't actually do anything. But we need the flags
+   * set for GTK to deliver events properly. */
+  gtk_widget_show (GTK_WIDGET (ui->frames));
+#else
   gtk_widget_realize (GTK_WIDGET (ui->frames));
+#endif
 
   g_object_set_data (G_OBJECT (gdisplay), "meta-ui", ui);
 
@@ -446,13 +469,10 @@ meta_ui_map_frame   (MetaUI *ui,
 {
   GdkWindow *window;
 
-#if GTK_CHECK_VERSION (3, 0, 0)
   GdkDisplay *display;
+
   display = gdk_x11_lookup_xdisplay (ui->xdisplay);
   window = gdk_x11_window_lookup_for_display (display, xwindow);
-#else
-  window = gdk_xid_table_lookup (xwindow);
-#endif
 
   if (window)
     gdk_window_show_unraised (window);
@@ -464,13 +484,10 @@ meta_ui_unmap_frame (MetaUI *ui,
 {
   GdkWindow *window;
 
-#if GTK_CHECK_VERSION (3, 0, 0)
   GdkDisplay *display;
+
   display = gdk_x11_lookup_xdisplay (ui->xdisplay);
   window = gdk_x11_window_lookup_for_display (display, xwindow);
-#else
-  window = gdk_xid_table_lookup (xwindow);
-#endif
 
   if (window)
     gdk_window_hide (window);
@@ -562,109 +579,6 @@ meta_ui_window_menu_free (MetaWindowMenu *menu)
   meta_window_menu_free (menu);
 }
 
-#if !GTK_CHECK_VERSION (3, 0, 0)
-struct _MetaImageWindow
-{
-  GtkWidget *window;
-  GdkPixmap *pixmap;
-};
-
-MetaImageWindow*
-meta_image_window_new (Display *xdisplay,
-                       int      screen_number,
-                       int      max_width,
-                       int      max_height)
-{
-  MetaImageWindow *iw;
-  GdkDisplay *gdisplay;
-  GdkScreen *gscreen;
-
-  iw = g_new (MetaImageWindow, 1);
-  iw->window = gtk_window_new (GTK_WINDOW_POPUP);
-
-  gdisplay = gdk_x11_lookup_xdisplay (xdisplay);
-  gscreen = gdk_display_get_screen (gdisplay, screen_number);
-
-  gtk_window_set_screen (GTK_WINDOW (iw->window), gscreen);
-
-  gtk_widget_realize (iw->window);
-  iw->pixmap = gdk_pixmap_new (gtk_widget_get_window (iw->window),
-                               max_width, max_height,
-                               -1);
-
-  gtk_widget_set_size_request (iw->window, 1, 1);
-  gtk_widget_set_double_buffered (iw->window, FALSE);
-  gtk_widget_set_app_paintable (iw->window, TRUE);
-
-  return iw;
-}
-
-void
-meta_image_window_free (MetaImageWindow *iw)
-{
-  gtk_widget_destroy (iw->window);
-  g_object_unref (G_OBJECT (iw->pixmap));
-  g_free (iw);
-}
-
-void
-meta_image_window_set_showing  (MetaImageWindow *iw,
-                                gboolean         showing)
-{
-  if (showing)
-    gtk_widget_show_all (iw->window);
-  else
-    {
-      gtk_widget_hide (iw->window);
-      meta_core_increment_event_serial (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()));
-    }
-}
-
-void
-meta_image_window_set (MetaImageWindow *iw,
-                       GdkPixbuf       *pixbuf,
-                       int              x,
-                       int              y)
-{
-#if GTK_CHECK_VERSION (3, 0, 0)
-  cairo_t *cr;
-#endif
-
-  /* We use a back pixmap to avoid having to handle exposes, because
-   * it's really too slow for large clients being minimized, etc.
-   * and this way flicker is genuinely zero.
-   */
-
-#if !GTK_CHECK_VERSION (3, 0, 0)
-  gdk_draw_pixbuf (iw->pixmap,
-                   gtk_widget_get_style (iw->window)->black_gc,
-                   pixbuf,
-                   0, 0,
-                   0, 0,
-                   gdk_pixbuf_get_width (pixbuf),
-                   gdk_pixbuf_get_height (pixbuf),
-                   GDK_RGB_DITHER_NORMAL,
-                   0, 0);
-#else
-  cr = gdk_cairo_create (iw->pixmap);
-  gdk_cairo_set_source_pixbuf (cr, pixbuf, 0, 0);
-  cairo_paint (cr);
-  cairo_destroy (cr);
-#endif
-
-  gdk_window_set_back_pixmap (gtk_widget_get_window (iw->window),
-                              iw->pixmap,
-                              FALSE);
-
-  gdk_window_move_resize (gtk_widget_get_window (iw->window),
-                          x, y,
-                          gdk_pixbuf_get_width (pixbuf),
-                          gdk_pixbuf_get_height (pixbuf));
-
-  gdk_window_clear (gtk_widget_get_window (iw->window));
-}
-#endif
-
 #if !GTK_CHECK_VERSION (3, 0, 0)
 static GdkColormap*
 get_cmap (GdkPixmap *pixmap)
@@ -693,7 +607,7 @@ get_cmap (GdkPixmap *pixmap)
 
   /* Be sure we aren't going to blow up due to visual mismatch */
   if (cmap &&
-      (gdk_colormap_get_visual (cmap)->depth !=
+      (gdk_visual_get_depth (gdk_colormap_get_visual (cmap)) !=
        gdk_drawable_get_depth (pixmap)))
     {
       cmap = NULL;
@@ -770,20 +684,12 @@ meta_gdk_pixbuf_get_from_pixmap (GdkPixbuf   *dest,
 {
   GdkDrawable *drawable;
   GdkPixbuf *retval;
-#if !GTK_CHECK_VERSION (3, 0, 0)
   GdkColormap *cmap;
-#endif
 
   retval = NULL;
-#if !GTK_CHECK_VERSION (3, 0, 0)
   cmap = NULL;
-#endif
 
-#if GTK_CHECK_VERSION (3, 0, 0)
   drawable = gdk_x11_window_lookup_for_display (gdk_display_get_default (), xpixmap);
-#else
-  drawable = gdk_xid_table_lookup (xpixmap);
-#endif
 
   if (drawable)
     g_object_ref (G_OBJECT (drawable));
@@ -801,10 +707,8 @@ meta_gdk_pixbuf_get_from_pixmap (GdkPixbuf   *dest,
                                              dest_x, dest_y,
                                              width, height);
     }
-#if !GTK_CHECK_VERSION (3, 0, 0)
   if (cmap)
     g_object_unref (G_OBJECT (cmap));
-#endif
   if (drawable)
     g_object_unref (G_OBJECT (drawable));
 
@@ -824,6 +728,20 @@ meta_ui_pop_delay_exposes  (MetaUI *ui)
   meta_frames_pop_delay_exposes (ui->frames);
 }
 
+static GdkPixbuf *
+load_default_window_icon (int size)
+{
+  GtkIconTheme *theme = gtk_icon_theme_get_default ();
+  const char *icon_name;
+
+  if (gtk_icon_theme_has_icon (theme, META_DEFAULT_ICON_NAME))
+    icon_name = META_DEFAULT_ICON_NAME;
+  else
+    icon_name = "image-missing";
+
+  return gtk_icon_theme_load_icon (theme, icon_name, size, 0, NULL);
+}
+
 GdkPixbuf*
 meta_ui_get_default_window_icon (MetaUI *ui)
 {
@@ -831,26 +749,7 @@ meta_ui_get_default_window_icon (MetaUI *ui)
 
   if (default_icon == NULL)
     {
-      GtkIconTheme *theme;
-      gboolean icon_exists;
-
-      theme = gtk_icon_theme_get_default ();
-
-      icon_exists = gtk_icon_theme_has_icon (theme, META_DEFAULT_ICON_NAME);
-
-      if (icon_exists)
-          default_icon = gtk_icon_theme_load_icon (theme,
-                                                   META_DEFAULT_ICON_NAME,
-                                                   META_ICON_WIDTH,
-                                                   0,
-                                                   NULL);
-      else
-          default_icon = gtk_icon_theme_load_icon (theme,
-                                                   "gtk-missing-image",
-                                                   META_ICON_WIDTH,
-                                                   0,
-                                                   NULL);
-
+      default_icon = load_default_window_icon (META_ICON_WIDTH);
       g_assert (default_icon);
     }
 
@@ -881,7 +780,7 @@ meta_ui_get_default_mini_icon (MetaUI *ui)
                                                    NULL);
       else
           default_icon = gtk_icon_theme_load_icon (theme,
-                                                   "gtk-missing-image",
+                                                   "image-missing",
                                                    META_MINI_ICON_WIDTH,
                                                    0,
                                                    NULL);
@@ -900,11 +799,10 @@ meta_ui_window_should_not_cause_focus (Display *xdisplay,
 {
   GdkWindow *window;
 
-#if GTK_CHECK_VERSION (3, 0, 0)
-  window = gdk_x11_window_lookup_for_display (gdk_display_get_default (), xwindow);
-#else
-  window = gdk_xid_table_lookup (xwindow);
-#endif
+  GdkDisplay *display;
+
+  display = gdk_x11_lookup_xdisplay (xdisplay);
+  window = gdk_x11_window_lookup_for_display (display, xwindow);
 
   /* we shouldn't cause focus if we're an override redirect
    * toplevel which is not foreign
@@ -915,48 +813,6 @@ meta_ui_window_should_not_cause_focus (Display *xdisplay,
     return FALSE;
 }
 
-char*
-meta_text_property_to_utf8 (Display             *xdisplay,
-                            const XTextProperty *prop)
-{
-#if GTK_CHECK_VERSION (3, 0, 0)
-  GdkDisplay *display;
-#endif
-  char **list;
-  int count;
-  char *retval;
-
-  list = NULL;
-
-#if GTK_CHECK_VERSION (3, 0, 0)
-  display = gdk_x11_lookup_xdisplay (xdisplay);
-  count = gdk_text_property_to_utf8_list_for_display (display,
-                                                      gdk_x11_xatom_to_atom_for_display (display, prop->encoding),
-                                                      prop->format,
-                                                      prop->value,
-                                                      prop->nitems,
-                                                      &list);
-#else
-  count = gdk_text_property_to_utf8_list (gdk_x11_xatom_to_atom (prop->encoding),
-                                          prop->format,
-                                          prop->value,
-                                          prop->nitems,
-                                          &list);
-#endif
-
-  if (count == 0)
-    retval = NULL;
-  else
-    {
-      retval = list[0];
-      list[0] = g_strdup (""); /* something to free */
-    }
-
-  g_strfreev (list);
-
-  return retval;
-}
-
 void
 meta_ui_theme_get_frame_borders (MetaUI *ui,
                                  MetaFrameType      type,
@@ -967,9 +823,12 @@ meta_ui_theme_get_frame_borders (MetaUI *ui,
                                  int               *right_width)
 {
   int text_height;
+#if GTK_CHECK_VERSION (3, 0, 0)
+  GtkStyleContext *style = NULL;
+  PangoFontDescription *free_font_desc = NULL;
+#endif
   PangoContext *context;
   const PangoFontDescription *font_desc;
-  GtkStyle *default_style;
 
   if (meta_ui_have_a_theme ())
     {
@@ -978,8 +837,26 @@ meta_ui_theme_get_frame_borders (MetaUI *ui,
 
       if (!font_desc)
         {
+#if GTK_CHECK_VERSION (3, 0, 0)
+          GdkDisplay *display = gdk_x11_lookup_xdisplay (ui->xdisplay);
+          GdkScreen *screen = gdk_display_get_screen (display, XScreenNumberOfScreen (ui->xscreen));
+          GtkWidgetPath *widget_path;
+
+          style = gtk_style_context_new ();
+          gtk_style_context_set_screen (style, screen);
+          widget_path = gtk_widget_path_new ();
+          gtk_widget_path_append_type (widget_path, GTK_TYPE_WINDOW);
+          gtk_style_context_set_path (style, widget_path);
+          gtk_widget_path_free (widget_path);
+
+          gtk_style_context_get (style, GTK_STATE_FLAG_NORMAL, "font", &free_font_desc, NULL);
+          font_desc = (const PangoFontDescription *) free_font_desc;
+#else
+          GtkStyle *default_style;
+
           default_style = gtk_widget_get_default_style ();
           font_desc = default_style->font_desc;
+#endif
         }
 
       text_height = meta_pango_font_desc_get_text_height (font_desc, context);
@@ -988,11 +865,21 @@ meta_ui_theme_get_frame_borders (MetaUI *ui,
                                     type, text_height, flags,
                                     top_height, bottom_height,
                                     left_width, right_width);
+
+#if GTK_CHECK_VERSION (3, 0, 0)
+      if (free_font_desc)
+        pango_font_description_free (free_font_desc);
+#endif
     }
   else
     {
       *top_height = *bottom_height = *left_width = *right_width = 0;
     }
+
+#if GTK_CHECK_VERSION (3, 0, 0)
+  if (style != NULL)
+    g_object_unref (style);
+#endif
 }
 
 void
@@ -1169,13 +1056,9 @@ meta_ui_window_is_widget (MetaUI *ui,
 {
   GdkWindow *window;
 
-#if GTK_CHECK_VERSION (3, 0, 0)
   GdkDisplay *display;
-  
+  display = gdk_x11_lookup_xdisplay (ui->xdisplay);
   window = gdk_x11_window_lookup_for_display (display, xwindow);
-#else
-  window = gdk_xid_table_lookup (xwindow);
-#endif
 
   if (window)
     {
@@ -1220,9 +1103,7 @@ GdkPixbuf* meta_ui_get_pixbuf_from_pixmap(Pixmap pmap)
 	GdkPixmap* gpmap;
 	GdkScreen* screen;
 	GdkPixbuf* pixbuf;
-#if !GTK_CHECK_VERSION (3, 0, 0)
 	GdkColormap* cmap;
-#endif
 	int width;
 	int height;
 	int depth;
@@ -1230,16 +1111,10 @@ GdkPixbuf* meta_ui_get_pixbuf_from_pixmap(Pixmap pmap)
 	gpmap = gdk_pixmap_foreign_new(pmap);
 	screen = gdk_drawable_get_screen(gpmap);
 
-#if GTK_CHECK_VERSION(3, 0, 0)
-	width = gdk_window_get_width(GDK_WINDOW(gpmap));
-	height = gdk_window_get_height(GDK_WINDOW(gpmap));
-#else
 	gdk_drawable_get_size(GDK_DRAWABLE(gpmap), &width, &height);
-#endif
 
 	depth = gdk_drawable_get_depth(GDK_DRAWABLE(gpmap));
 
-#if !GTK_CHECK_VERSION (3, 0, 0)
 	if (depth <= 24)
 	{
 		cmap = gdk_screen_get_system_colormap(screen);
@@ -1248,7 +1123,6 @@ GdkPixbuf* meta_ui_get_pixbuf_from_pixmap(Pixmap pmap)
 	{
 		cmap = gdk_screen_get_rgba_colormap(screen);
 	}
-#endif
 
 	pixbuf = gdk_pixbuf_get_from_drawable(NULL, gpmap, cmap, 0, 0, 0, 0, width, height);
 
